// parse_cfg.cpp          -*-c++-*-
// written by: Laurentiu Pancescu

#include <stdlib.h>
#include <iostream.h>
#include <fstream.h>
#include <string>
#include <vector>
#include <iterator>

#include "parse_cfg.h"
#include "constants.h"
#include "line.h"

struct db
{
  db() : name("dummy"), var(dummy) {}
  db(const string &n, double &v):name(n), var(v) {}
  db(const db &d):name(d.name), var(d.var) {}
  bool operator==(const db &d) const { return name == d.name; }
  bool operator==(const string &s) const { return name == s; }
  db &operator=(const db &d) { name = d.name; var = d.var; return *this; }
  string name;
  double &var;
  static double dummy;
};

ostream &operator<<(ostream &os, const db &db_elem)
{
  os << prg_name << ": cfg: " 
     << db_elem.name << " = " << db_elem.var;
  return os;
}

void parse_cfg()
{
  vector<db> database;
  database.push_back(db("q", q));
  database.push_back(db("epsilons", epsilons));
  database.push_back(db("C0", C0));
  database.push_back(db("Vfb", Vfb));
  database.push_back(db("Ec", Ec));
  database.push_back(db("Ev", Ev));
  database.push_back(db("sigma", sigma));
  database.push_back(db("k", k));
  database.push_back(db("T", T));
  database.push_back(db("T0", T0));
  database.push_back(db("Ea", Ea));
  database.push_back(db("NA", NA));
  database.push_back(db("g", g));
  database.push_back(db("C", C));
  database.push_back(db("xmax", xmax));
  database.push_back(db("cc", cc));
  database.push_back(db("W", W));
  database.push_back(db("L", L));
  database.push_back(db("Vd", Vd));
  database.push_back(db("v0_max", v0_max));
  database.push_back(db("num_prec", num_prec));
  ifstream cfg_file("cris3cfg.txt");
  if (!cfg_file)
    {
      cerr << prg_name << ": unable to open \"cris3cfg.txt\"\n";
      return;
    }
  cerr << line(70, '-') << '\n'
       << prg_name << ": cris3cfg.txt: start parsing\n";
  string current_line;
  unsigned line_no = 0;
  while (getline(cfg_file, current_line))
    {
      ++line_no;
      string::size_type eq = current_line.find("=");
      if (eq == string::npos)
        {
          if (current_line.size())
            cerr << prg_name 
                 << ": cris3cfg.txt: warning: ignoring invalid line "
                 << line_no << '\n';
          continue;
        }
      string name = current_line.substr(0, eq);
      string value = current_line.substr(eq + 1, current_line.size() - eq - 1);
      bool found = false;
      vector<db>::const_iterator ci;
      for (ci = database.begin(); ci != database.end(); ci++)
        if (ci -> name == name)
          {
            found = true;
            break;
          }
      if (!found)
        {
          cerr << prg_name
               << ": cris3cfg.txt: warning: ignoring unknown parameter \""
               << name << "\" on line " << line_no << '\n';
          continue;
        }
      cerr << prg_name << ": cris3cfg.txt: " << name << " = "
           << (ci -> var = atof(value.c_str())) << endl;
    }
  cerr << "cris3: cris3cfg.txt: end parsing\n" << line(70, '-') << '\n';
  copy(database.begin(), database.end(), ostream_iterator<db>(cerr, "\n"));
  cerr << line(70, '-') << '\n';
}
